# Accumulate 1.1

## Executor System

1.1 introduces a new version of the executor system. Where executor v1 was built
around the idea of a 'delivery' containing a transaction plus its signatures,
executor v2 is built around 'messages'. A more complete description of the
messaging model can be found in the [core docs][core-docs] but in essence the
processing system has been rearranged to individually process transactions,
signatures, and other types of messages.

Note, it is not *completely* true that messages are processed individually. When
an envelope is received, the envelope is converted into a message bundle, which
is then processed. The executor checks that there is at least one signature for
every transaction in the bundle. Additionally, message executors may look in the
bundle for a message before resorting to a database query, reducing disk I/O and
simplifying message execution.

A message executor may call another message executor if the message being
processed contains or references another message. For example, a signature
references a transaction, so the signature executor may call the transaction
executor. And a synthetic message wraps another message so the synthetic message
executor may call another message executor for the wrapped message.

The changes to signatures and system messages are discussed below. Beginning a
block, ending a block, and processing a transaction have not changed
significantly.

[core-docs]: https://docs.accumulatenetwork.io/core/

### Signatures

While the initial processing of a signature has not changed much (though it has
been rewritten), the process by which a signature is 'applied to' a transaction
has been completely reworked. The only significant difference in initial
processing relates to how fees are paid. A signature is included in a block as
long as the signer has a non-zero credit balance, even if the balance is not
sufficient to pay for the transaction. When the signature is executed, if the
balance is not sufficient a minimum fee is deducted and the signature is
recorded and rejected. Signatures must still pass the other validation checks
(the signer must exist, the public key must match, etc).

In executor v2, a user signature[^1] is a message from the user (alternatively,
from the user's wallet) to an authority (e.g. a key book). Signatures are
processed within the context of the signer's authority and never interact
directly with the transaction or its principal account. Instead, once the
authority's signing conditions are met (such as a threshold), the authority
sends an authority signature as a synthetic message. If the authority is acting
as a direct (non-delegated) authority, the authority signature is sent to the
transaction's principal account and counted as a 'vote' for the transaction. The
transaction is executed once all of the principal's active authorities have
voted in favor of the transaction. This is in contrast to executor v1 which
applies signatures directly to transactions, forwarding the signature itself if
necessary.

When an authority is acting as a delegated authority, the authority signature is
sent to the delegator instead of the transaction principal. The delegator counts
the authority signature as a signature on behalf of the delegate's entry in the
signer. The delegator follows the same process, issuing an authority signature
once its conditions are met. This process continues along the delegation chain
until an authority signature is sent to the principal.

[^1]: The term 'user signature' is used to differentiate normal key signatures
    (included delegated signatures) from authority signatures, which are
    produced by the protocol (aka are synthetic).

### System Messages

The security model of system messages is unchanged but the specifics have. The
security of a synthetic message is provided by a Merkle receipt. Executor v1
allowed the receipt to be split across multiple envelopes as allowed the latency
of synthetic messages to be reduced, though this required recording synthetic
messages on a pending list until the full receipt was received. However pending
synthetic messages caused significant performance issues and so this capability
was disabled for MainNet and has been completely removed in executor v2. Thus
executor v2 will reject synthetic messages unless they are accompanied with a
complete receipt that ends in a trusted DN root chain anchor.

Accumulate 1.0 transmitted synthetic transactions in the same way as user
transactions along with a special type of signature containing the receipt.
Accumulate 1.1 removes support for this signature type and instead defines a
specific message type for synthetic messages which includes the receipt and the
message. As stated above, this receipt must be complete - it must begin at the
message's hash and end at a trusted DN root chain anchor.

Accumulate 1.0 transmitted anchor transactions in the same way as user
transactions along with a plain key signature signed with the validator key.
However that signature required special processing rules as it was signed by a
validator key instead of a key from a key page or lite identity. Accumulate 1.1
removes support for validator key signatures from the signature message executor
and instead defines a specific message type for anchor transactions which
includes the anchor and signature. The signature is processed directly by the
anchor message executor.

Anchor transactions and synthetic messages must be sequenced. Accumulate 1.0
transmitted sequence metadata via a special type of signature. This required
special processing rules in the signature processing code and increased the
complexity of transaction processing. Accumulate 1.1 removes support for this
signature type and instead defines a specific message type for sequenced
messages which includes the message and sequence metadata. This allows all the
sequence handling code to be removed from other executors and located
exclusively in the sequenced message executor.

In Accumulate 1.1, anchors are transmitted as an anchor message, which contains
a signature and a sequenced message, which contains the transaction as a
transaction message. And synthetic messages are transmitted as a synthetic
message, which contains a receipt and a sequenced message, which contains the
produced message, which may be a transaction, authority signature, etc.

## API

TODO