// Copyright 2025 The Accumulate Authors
//
// Use of this source code is governed by an MIT-style
// license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.

package api

// GENERATED BY go run ./tools/cmd/gen-enum. DO NOT EDIT.

import (
	"encoding/json"
	"fmt"
	"strings"
)

// EventTypeError .
const EventTypeError EventType = 1

// EventTypeBlock .
const EventTypeBlock EventType = 2

// EventTypeGlobals .
const EventTypeGlobals EventType = 3

// PeerStatusIsUnknown .
const PeerStatusIsUnknown KnownPeerStatus = 0

// PeerStatusIsKnownGood .
const PeerStatusIsKnownGood KnownPeerStatus = 1

// PeerStatusIsKnownBad .
const PeerStatusIsKnownBad KnownPeerStatus = 2

// QueryTypeDefault .
const QueryTypeDefault QueryType = 0

// QueryTypeChain .
const QueryTypeChain QueryType = 1

// QueryTypeData .
const QueryTypeData QueryType = 2

// QueryTypeDirectory .
const QueryTypeDirectory QueryType = 3

// QueryTypePending .
const QueryTypePending QueryType = 4

// QueryTypeBlock .
const QueryTypeBlock QueryType = 5

// QueryTypeAnchorSearch .
const QueryTypeAnchorSearch QueryType = 16

// QueryTypePublicKeySearch .
const QueryTypePublicKeySearch QueryType = 17

// QueryTypePublicKeyHashSearch .
const QueryTypePublicKeyHashSearch QueryType = 18

// QueryTypeDelegateSearch .
const QueryTypeDelegateSearch QueryType = 19

// QueryTypeMessageHashSearch .
const QueryTypeMessageHashSearch QueryType = 20

// RecordTypeAccount .
const RecordTypeAccount RecordType = 1

// RecordTypeChain .
const RecordTypeChain RecordType = 2

// RecordTypeChainEntry .
const RecordTypeChainEntry RecordType = 3

// RecordTypeKey .
const RecordTypeKey RecordType = 4

// RecordTypeMessage .
const RecordTypeMessage RecordType = 16

// RecordTypeSignatureSet .
const RecordTypeSignatureSet RecordType = 17

// RecordTypeMinorBlock .
const RecordTypeMinorBlock RecordType = 32

// RecordTypeMajorBlock .
const RecordTypeMajorBlock RecordType = 33

// RecordTypeRange .
const RecordTypeRange RecordType = 128

// RecordTypeUrl .
const RecordTypeUrl RecordType = 129

// RecordTypeTxID .
const RecordTypeTxID RecordType = 130

// RecordTypeIndexEntry .
const RecordTypeIndexEntry RecordType = 131

// RecordTypeError .
const RecordTypeError RecordType = 143

// ServiceTypeUnknown indicates an unknown service type.
const ServiceTypeUnknown ServiceType = 0

// ServiceTypeNode is the type of [NodeService].
const ServiceTypeNode ServiceType = 1

// ServiceTypeConsensus is the type of [ConsensusService].
const ServiceTypeConsensus ServiceType = 2

// ServiceTypeNetwork is the type of [NetworkService].
const ServiceTypeNetwork ServiceType = 3

// ServiceTypeMetrics is the type of [MetricsService].
const ServiceTypeMetrics ServiceType = 4

// ServiceTypeQuery is the type of [Querier].
const ServiceTypeQuery ServiceType = 5

// ServiceTypeEvent is the type of [EventService].
const ServiceTypeEvent ServiceType = 6

// ServiceTypeSubmit is the type of [Submitter].
const ServiceTypeSubmit ServiceType = 7

// ServiceTypeValidate is the type of [Validator].
const ServiceTypeValidate ServiceType = 8

// ServiceTypeFaucet is the type of [Faucet].
const ServiceTypeFaucet ServiceType = 9

// ServiceTypeSnapshot is the type of [SnapshotService].
const ServiceTypeSnapshot ServiceType = 10

// GetEnumValue returns the value of the Event Type
func (v EventType) GetEnumValue() uint64 { return uint64(v) }

// SetEnumValue sets the value. SetEnumValue returns false if the value is invalid.
func (v *EventType) SetEnumValue(id uint64) bool {
	u := EventType(id)
	switch u {
	case EventTypeError, EventTypeBlock, EventTypeGlobals:
		*v = u
		return true
	}
	return false
}

// String returns the name of the Event Type.
func (v EventType) String() string {
	switch v {
	case EventTypeError:
		return "error"
	case EventTypeBlock:
		return "block"
	case EventTypeGlobals:
		return "globals"
	}
	return fmt.Sprintf("EventType:%d", v)
}

// EventTypeByName returns the named Event Type.
func EventTypeByName(name string) (EventType, bool) {
	switch strings.ToLower(name) {
	case "error":
		return EventTypeError, true
	case "block":
		return EventTypeBlock, true
	case "globals":
		return EventTypeGlobals, true
	}
	return 0, false
}

// MarshalJSON marshals the Event Type to JSON as a string.
func (v EventType) MarshalJSON() ([]byte, error) {
	return json.Marshal(v.String())
}

// UnmarshalJSON unmarshals the Event Type from JSON as a string.
func (v *EventType) UnmarshalJSON(data []byte) error {
	var s string
	err := json.Unmarshal(data, &s)
	if err != nil {
		return err
	}

	var ok bool
	*v, ok = EventTypeByName(s)
	if !ok || strings.ContainsRune(v.String(), ':') {
		return fmt.Errorf("invalid Event Type %q", s)
	}
	return nil
}

// GetEnumValue returns the value of the Known Peer Status
func (v KnownPeerStatus) GetEnumValue() uint64 { return uint64(v) }

// SetEnumValue sets the value. SetEnumValue returns false if the value is invalid.
func (v *KnownPeerStatus) SetEnumValue(id uint64) bool {
	u := KnownPeerStatus(id)
	switch u {
	case PeerStatusIsUnknown, PeerStatusIsKnownGood, PeerStatusIsKnownBad:
		*v = u
		return true
	}
	return false
}

// String returns the name of the Known Peer Status.
func (v KnownPeerStatus) String() string {
	switch v {
	case PeerStatusIsUnknown:
		return "unknown"
	case PeerStatusIsKnownGood:
		return "good"
	case PeerStatusIsKnownBad:
		return "bad"
	}
	return fmt.Sprintf("KnownPeerStatus:%d", v)
}

// KnownPeerStatusByName returns the named Known Peer Status.
func KnownPeerStatusByName(name string) (KnownPeerStatus, bool) {
	switch strings.ToLower(name) {
	case "unknown":
		return PeerStatusIsUnknown, true
	case "good":
		return PeerStatusIsKnownGood, true
	case "bad":
		return PeerStatusIsKnownBad, true
	}
	return 0, false
}

// MarshalJSON marshals the Known Peer Status to JSON as a string.
func (v KnownPeerStatus) MarshalJSON() ([]byte, error) {
	return json.Marshal(v.String())
}

// UnmarshalJSON unmarshals the Known Peer Status from JSON as a string.
func (v *KnownPeerStatus) UnmarshalJSON(data []byte) error {
	var s string
	err := json.Unmarshal(data, &s)
	if err != nil {
		return err
	}

	var ok bool
	*v, ok = KnownPeerStatusByName(s)
	if !ok || strings.ContainsRune(v.String(), ':') {
		return fmt.Errorf("invalid Known Peer Status %q", s)
	}
	return nil
}

// GetEnumValue returns the value of the Query Type
func (v QueryType) GetEnumValue() uint64 { return uint64(v) }

// SetEnumValue sets the value. SetEnumValue returns false if the value is invalid.
func (v *QueryType) SetEnumValue(id uint64) bool {
	u := QueryType(id)
	switch u {
	case QueryTypeDefault, QueryTypeChain, QueryTypeData, QueryTypeDirectory, QueryTypePending, QueryTypeBlock, QueryTypeAnchorSearch, QueryTypePublicKeySearch, QueryTypePublicKeyHashSearch, QueryTypeDelegateSearch, QueryTypeMessageHashSearch:
		*v = u
		return true
	}
	return false
}

// String returns the name of the Query Type.
func (v QueryType) String() string {
	switch v {
	case QueryTypeDefault:
		return "default"
	case QueryTypeChain:
		return "chain"
	case QueryTypeData:
		return "data"
	case QueryTypeDirectory:
		return "directory"
	case QueryTypePending:
		return "pending"
	case QueryTypeBlock:
		return "block"
	case QueryTypeAnchorSearch:
		return "anchorSearch"
	case QueryTypePublicKeySearch:
		return "publicKeySearch"
	case QueryTypePublicKeyHashSearch:
		return "publicKeyHashSearch"
	case QueryTypeDelegateSearch:
		return "delegateSearch"
	case QueryTypeMessageHashSearch:
		return "messageHashSearch"
	}
	return fmt.Sprintf("QueryType:%d", v)
}

// QueryTypeByName returns the named Query Type.
func QueryTypeByName(name string) (QueryType, bool) {
	switch strings.ToLower(name) {
	case "default":
		return QueryTypeDefault, true
	case "chain":
		return QueryTypeChain, true
	case "data":
		return QueryTypeData, true
	case "directory":
		return QueryTypeDirectory, true
	case "pending":
		return QueryTypePending, true
	case "block":
		return QueryTypeBlock, true
	case "anchorsearch":
		return QueryTypeAnchorSearch, true
	case "publickeysearch":
		return QueryTypePublicKeySearch, true
	case "publickeyhashsearch":
		return QueryTypePublicKeyHashSearch, true
	case "delegatesearch":
		return QueryTypeDelegateSearch, true
	case "messagehashsearch":
		return QueryTypeMessageHashSearch, true
	}
	return 0, false
}

// MarshalJSON marshals the Query Type to JSON as a string.
func (v QueryType) MarshalJSON() ([]byte, error) {
	return json.Marshal(v.String())
}

// UnmarshalJSON unmarshals the Query Type from JSON as a string.
func (v *QueryType) UnmarshalJSON(data []byte) error {
	var s string
	err := json.Unmarshal(data, &s)
	if err != nil {
		return err
	}

	var ok bool
	*v, ok = QueryTypeByName(s)
	if !ok || strings.ContainsRune(v.String(), ':') {
		return fmt.Errorf("invalid Query Type %q", s)
	}
	return nil
}

// GetEnumValue returns the value of the Record Type
func (v RecordType) GetEnumValue() uint64 { return uint64(v) }

// SetEnumValue sets the value. SetEnumValue returns false if the value is invalid.
func (v *RecordType) SetEnumValue(id uint64) bool {
	u := RecordType(id)
	switch u {
	case RecordTypeAccount, RecordTypeChain, RecordTypeChainEntry, RecordTypeKey, RecordTypeMessage, RecordTypeSignatureSet, RecordTypeMinorBlock, RecordTypeMajorBlock, RecordTypeRange, RecordTypeUrl, RecordTypeTxID, RecordTypeIndexEntry, RecordTypeError:
		*v = u
		return true
	}
	return false
}

// String returns the name of the Record Type.
func (v RecordType) String() string {
	switch v {
	case RecordTypeAccount:
		return "account"
	case RecordTypeChain:
		return "chain"
	case RecordTypeChainEntry:
		return "chainEntry"
	case RecordTypeKey:
		return "key"
	case RecordTypeMessage:
		return "message"
	case RecordTypeSignatureSet:
		return "signatureSet"
	case RecordTypeMinorBlock:
		return "minorBlock"
	case RecordTypeMajorBlock:
		return "majorBlock"
	case RecordTypeRange:
		return "range"
	case RecordTypeUrl:
		return "url"
	case RecordTypeTxID:
		return "txID"
	case RecordTypeIndexEntry:
		return "indexEntry"
	case RecordTypeError:
		return "error"
	}
	return fmt.Sprintf("RecordType:%d", v)
}

// RecordTypeByName returns the named Record Type.
func RecordTypeByName(name string) (RecordType, bool) {
	switch strings.ToLower(name) {
	case "account":
		return RecordTypeAccount, true
	case "chain":
		return RecordTypeChain, true
	case "chainentry":
		return RecordTypeChainEntry, true
	case "key":
		return RecordTypeKey, true
	case "message":
		return RecordTypeMessage, true
	case "signatureset":
		return RecordTypeSignatureSet, true
	case "minorblock":
		return RecordTypeMinorBlock, true
	case "majorblock":
		return RecordTypeMajorBlock, true
	case "range":
		return RecordTypeRange, true
	case "url":
		return RecordTypeUrl, true
	case "txid":
		return RecordTypeTxID, true
	case "indexentry":
		return RecordTypeIndexEntry, true
	case "error":
		return RecordTypeError, true
	}
	return 0, false
}

// MarshalJSON marshals the Record Type to JSON as a string.
func (v RecordType) MarshalJSON() ([]byte, error) {
	return json.Marshal(v.String())
}

// UnmarshalJSON unmarshals the Record Type from JSON as a string.
func (v *RecordType) UnmarshalJSON(data []byte) error {
	var s string
	err := json.Unmarshal(data, &s)
	if err != nil {
		return err
	}

	var ok bool
	*v, ok = RecordTypeByName(s)
	if !ok || strings.ContainsRune(v.String(), ':') {
		return fmt.Errorf("invalid Record Type %q", s)
	}
	return nil
}

// GetEnumValue returns the value of the Service Type
func (v ServiceType) GetEnumValue() uint64 { return uint64(v) }

// SetEnumValue sets the value. SetEnumValue returns false if the value is invalid.
func (v *ServiceType) SetEnumValue(id uint64) bool {
	u := ServiceType(id)
	switch u {
	case ServiceTypeUnknown, ServiceTypeNode, ServiceTypeConsensus, ServiceTypeNetwork, ServiceTypeMetrics, ServiceTypeQuery, ServiceTypeEvent, ServiceTypeSubmit, ServiceTypeValidate, ServiceTypeFaucet, ServiceTypeSnapshot:
		*v = u
		return true
	}
	return false
}

// String returns the name of the Service Type.
func (v ServiceType) String() string {
	switch v {
	case ServiceTypeUnknown:
		return "unknown"
	case ServiceTypeNode:
		return "node"
	case ServiceTypeConsensus:
		return "consensus"
	case ServiceTypeNetwork:
		return "network"
	case ServiceTypeMetrics:
		return "metrics"
	case ServiceTypeQuery:
		return "query"
	case ServiceTypeEvent:
		return "event"
	case ServiceTypeSubmit:
		return "submit"
	case ServiceTypeValidate:
		return "validate"
	case ServiceTypeFaucet:
		return "faucet"
	case ServiceTypeSnapshot:
		return "snapshot"
	}
	return fmt.Sprintf("ServiceType:%d", v)
}

// ServiceTypeByName returns the named Service Type.
func ServiceTypeByName(name string) (ServiceType, bool) {
	switch strings.ToLower(name) {
	case "unknown":
		return ServiceTypeUnknown, true
	case "node":
		return ServiceTypeNode, true
	case "consensus":
		return ServiceTypeConsensus, true
	case "network":
		return ServiceTypeNetwork, true
	case "metrics":
		return ServiceTypeMetrics, true
	case "query":
		return ServiceTypeQuery, true
	case "event":
		return ServiceTypeEvent, true
	case "submit":
		return ServiceTypeSubmit, true
	case "validate":
		return ServiceTypeValidate, true
	case "faucet":
		return ServiceTypeFaucet, true
	case "snapshot":
		return ServiceTypeSnapshot, true
	}
	return 0, false
}

// MarshalJSON marshals the Service Type to JSON as a string.
func (v ServiceType) MarshalJSON() ([]byte, error) {
	return json.Marshal(v.String())
}

// UnmarshalJSON unmarshals the Service Type from JSON as a string.
func (v *ServiceType) UnmarshalJSON(data []byte) error {
	var s string
	err := json.Unmarshal(data, &s)
	if err != nil {
		return err
	}

	var ok bool
	*v, ok = ServiceTypeByName(s)
	if !ok || strings.ContainsRune(v.String(), ':') {
		return fmt.Errorf("invalid Service Type %q", s)
	}
	return nil
}
