// Copyright 2022 The Accumulate Authors
//
// Use of this source code is governed by an MIT-style
// license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.

package p2p

// GENERATED BY go run ./tools/cmd/gen-types. DO NOT EDIT.

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io"

	"gitlab.com/accumulatenetwork/accumulate/pkg/types/encoding"
)

// newEvent creates a new event for the specified eventType.
func newEvent(typ eventType) (event, error) {
	switch typ {
	case eventTypeServiceRegistered:
		return new(serviceRegisteredEvent), nil
	}
	return nil, fmt.Errorf("unknown event %v", typ)
}

// equalEvent is used to compare the values of the union
func equalEvent(a, b event) bool {
	if a == b {
		return true
	}
	switch a := a.(type) {
	case *serviceRegisteredEvent:
		if a == nil {
			return b == nil
		}
		b, ok := b.(*serviceRegisteredEvent)
		return ok && a.Equal(b)
	}
	return false
}

// copyEvent copies a event.
func copyEvent(v event) event {
	switch v := v.(type) {
	case *serviceRegisteredEvent:
		return v.Copy()
	default:
		return v.CopyAsInterface().(event)
	}
}

// unmarshalEvent unmarshals a event.
func unmarshalEvent(data []byte) (event, error) {
	return unmarshalEventFrom(bytes.NewReader(data))
}

// unmarshalEventFrom unmarshals a event.
func unmarshalEventFrom(rd io.Reader) (event, error) {
	reader := encoding.NewReader(rd)

	// Read the type code
	var typ eventType
	if !reader.ReadEnum(1, &typ) {
		if reader.IsEmpty() {
			return nil, nil
		}
		return nil, fmt.Errorf("field Type: missing")
	}

	// Create a new event
	v, err := newEvent(eventType(typ))
	if err != nil {
		return nil, err
	}

	// Unmarshal the rest of the event
	err = v.UnmarshalFieldsFrom(reader)
	if err != nil {
		return nil, err
	}

	return v, nil
}

// unmarshalEventJson unmarshals a event.
func unmarshalEventJSON(data []byte) (event, error) {
	var typ *struct{ Type eventType }
	err := json.Unmarshal(data, &typ)
	if err != nil {
		return nil, err
	}

	if typ == nil {
		return nil, nil
	}

	acnt, err := newEvent(typ.Type)
	if err != nil {
		return nil, err
	}

	err = json.Unmarshal(data, acnt)
	if err != nil {
		return nil, err
	}

	return acnt, nil
}
