// Copyright 2022 The Accumulate Authors
//
// Use of this source code is governed by an MIT-style
// license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.

package bpt

// GENERATED BY go run ./tools/cmd/gen-model. DO NOT EDIT.

//lint:file-ignore S1008,U1000 generated code

import (
	"gitlab.com/accumulatenetwork/accumulate/internal/logging"
	record "gitlab.com/accumulatenetwork/accumulate/pkg/database"
	"gitlab.com/accumulatenetwork/accumulate/pkg/database/values"
)

type BPT struct {
	logger      logging.OptionalLogger
	store       record.Store
	key         *record.Key
	pending     map[[32]byte]*mutation
	loadedState *stateData

	state values.Value[*stateData]
	root  *rootRecord
}

func (c *BPT) Key() *record.Key { return c.key }

func (c *BPT) getState() values.Value[*stateData] {
	return values.GetOrCreate(c, &c.state, (*BPT).newState)
}

func (c *BPT) newState() values.Value[*stateData] {
	return values.NewValue(c.logger.L, c.store, c.key.Append("Root"), false, values.Struct[stateData]())
}

func (c *BPT) baseIsDirty() bool {
	if c == nil {
		return false
	}

	if values.IsDirty(c.state) {
		return true
	}
	if values.IsDirty(c.root) {
		return true
	}

	return false
}

func (c *BPT) baseCommit() error {
	if c == nil {
		return nil
	}

	var err error
	values.Commit(&err, c.state)
	values.Commit(&err, c.root)

	return err
}
